from .__init__ import ctypes, typing, abc, \
    SDL_POINTER, SDL_ENUM, SDL_FUNC, SDL_TYPE, SDL_BINARY

from .SDL_events import SDL_Event
from .SDL_video import SDL_Window, SDL_WindowFlags
from .SDL_rect import SDL_FPoint, SDL_Rect, SDL_FRect
from .SDL_surface import SDL_Surface, SDL_ScaleMode, SDL_FlipMode
from .SDL_pixels import SDL_FColor, SDL_PixelFormat
from .SDL_properties import SDL_PropertiesID
from .SDL_blendmode import SDL_BlendMode

SDL_SOFTWARE_RENDERER: bytes = "software".encode()

class SDL_Vertex(ctypes.Structure):
    _fields_ = [
        ("position", SDL_FPoint),
        ("color", SDL_FColor),
        ("tex_coord", SDL_FPoint)
    ]

SDL_TextureAccess: typing.TypeAlias = SDL_TYPE["SDL_TextureAccess", SDL_ENUM]

SDL_TEXTUREACCESS_STATIC, SDL_TEXTUREACCESS_STREAMING, SDL_TEXTUREACCESS_TARGET = range(3)

SDL_RendererLogicalPresentation: typing.TypeAlias = SDL_TYPE["SDL_RendererLogicalPresentation", SDL_ENUM]

SDL_LOGICAL_PRESENTATION_DISABLED, SDL_LOGICAL_PRESENTATION_STRETCH, SDL_LOGICAL_PRESENTATION_LETTERBOX, \
    SDL_LOGICAL_PRESENTATION_OVERSCAN, SDL_LOGICAL_PRESENTATION_INTEGER_SCALE = range(5)

class SDL_Renderer(ctypes.c_void_p):
    ...

class SDL_Texture(ctypes.Structure):
    _fields_ = [
        ("format", SDL_PixelFormat),
        ("w", ctypes.c_int),
        ("h", ctypes.c_int),
        ("refcount", ctypes.c_int)
    ]

SDL_GetNumRenderDrivers: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetNumRenderDrivers", ctypes.c_int, [], SDL_BINARY]
SDL_GetRenderDriver: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderDriver", ctypes.c_char_p, [ctypes.c_int], SDL_BINARY]

SDL_CreateWindowAndRenderer: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateWindowAndRenderer", ctypes.c_bool, [ctypes.c_char_p, ctypes.c_int, ctypes.c_int, SDL_WindowFlags, SDL_POINTER[SDL_POINTER[SDL_Window]], SDL_POINTER[SDL_POINTER[SDL_Renderer]]], SDL_BINARY]
SDL_CreateRenderer: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateRenderer", SDL_POINTER[SDL_Renderer], [SDL_POINTER[SDL_Window], ctypes.c_char_p], SDL_BINARY]
SDL_CreateRendererWithProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateRendererWithProperties", SDL_POINTER[SDL_Renderer], [SDL_PropertiesID], SDL_BINARY]

SDL_PROP_RENDERER_CREATE_NAME_STRING: bytes = "SDL.renderer.create.name".encode()
SDL_PROP_RENDERER_CREATE_WINDOW_POINTER: bytes = "SDL.renderer.create.window".encode()
SDL_PROP_RENDERER_CREATE_SURFACE_POINTER: bytes = "SDL.renderer.create.surface".encode()
SDL_PROP_RENDERER_CREATE_OUTPUT_COLORSPACE_NUMBER: bytes = "SDL.renderer.create.output_colorspace".encode()
SDL_PROP_RENDERER_CREATE_PRESENT_VSYNC_NUMBER: bytes = "SDL.renderer.create.present_vsync".encode()
SDL_PROP_RENDERER_CREATE_VULKAN_INSTANCE_POINTER: bytes = "SDL.renderer.create.vulkan.instance".encode()
SDL_PROP_RENDERER_CREATE_VULKAN_SURFACE_NUMBER: bytes = "SDL.renderer.create.vulkan.surface".encode()
SDL_PROP_RENDERER_CREATE_VULKAN_PHYSICAL_DEVICE_POINTER: bytes = "SDL.renderer.create.vulkan.physical_device".encode()
SDL_PROP_RENDERER_CREATE_VULKAN_DEVICE_POINTER: bytes = "SDL.renderer.create.vulkan.device".encode()
SDL_PROP_RENDERER_CREATE_VULKAN_GRAPHICS_QUEUE_FAMILY_INDEX_NUMBER: bytes = "SDL.renderer.create.vulkan.graphics_queue_family_index".encode()
SDL_PROP_RENDERER_CREATE_VULKAN_PRESENT_QUEUE_FAMILY_INDEX_NUMBER: bytes = "SDL.renderer.create.vulkan.present_queue_family_index".encode()

SDL_CreateSoftwareRenderer: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateSoftwareRenderer", SDL_POINTER[SDL_Renderer], [SDL_POINTER[SDL_Surface]], SDL_BINARY]
SDL_GetRenderer: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderer", SDL_POINTER[SDL_Renderer], [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_GetRenderWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderWindow", SDL_POINTER[SDL_Window], [SDL_POINTER[SDL_Renderer]], SDL_BINARY]
SDL_GetRendererName: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRendererName", ctypes.c_char_p, [SDL_POINTER[SDL_Renderer]], SDL_BINARY]

SDL_GetRendererProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRendererProperties", SDL_PropertiesID, [SDL_POINTER[SDL_Renderer]], SDL_BINARY]

SDL_PROP_RENDERER_NAME_STRING: bytes = "SDL.renderer.name".encode()
SDL_PROP_RENDERER_WINDOW_POINTER: bytes = "SDL.renderer.window".encode()
SDL_PROP_RENDERER_SURFACE_POINTER: bytes = "SDL.renderer.surface".encode()
SDL_PROP_RENDERER_VSYNC_NUMBER: bytes = "SDL.renderer.vsync".encode()
SDL_PROP_RENDERER_MAX_TEXTURE_SIZE_NUMBER: bytes = "SDL.renderer.max_texture_size".encode()
SDL_PROP_RENDERER_TEXTURE_FORMATS_POINTER: bytes = "SDL.renderer.texture_formats".encode()
SDL_PROP_RENDERER_OUTPUT_COLORSPACE_NUMBER: bytes = "SDL.renderer.output_colorspace".encode()
SDL_PROP_RENDERER_HDR_ENABLED_BOOLEAN: bytes = "SDL.renderer.HDR_enabled".encode()
SDL_PROP_RENDERER_SDR_WHITE_POINT_FLOAT: bytes = "SDL.renderer.SDR_white_point".encode()
SDL_PROP_RENDERER_HDR_HEADROOM_FLOAT: bytes = "SDL.renderer.HDR_headroom".encode()
SDL_PROP_RENDERER_D3D9_DEVICE_POINTER: bytes = "SDL.renderer.d3d9.device".encode()
SDL_PROP_RENDERER_D3D11_DEVICE_POINTER: bytes = "SDL.renderer.d3d11.device".encode()
SDL_PROP_RENDERER_D3D11_SWAPCHAIN_POINTER: bytes = "SDL.renderer.d3d11.swap_chain".encode()
SDL_PROP_RENDERER_D3D12_DEVICE_POINTER: bytes = "SDL.renderer.d3d12.device".encode()
SDL_PROP_RENDERER_D3D12_SWAPCHAIN_POINTER: bytes = "SDL.renderer.d3d12.swap_chain".encode()
SDL_PROP_RENDERER_D3D12_COMMAND_QUEUE_POINTER: bytes = "SDL.renderer.d3d12.command_queue".encode()
SDL_PROP_RENDERER_VULKAN_INSTANCE_POINTER: bytes = "SDL.renderer.vulkan.instance".encode()
SDL_PROP_RENDERER_VULKAN_SURFACE_NUMBER: bytes = "SDL.renderer.vulkan.surface".encode()
SDL_PROP_RENDERER_VULKAN_PHYSICAL_DEVICE_POINTER: bytes = "SDL.renderer.vulkan.physical_device".encode()
SDL_PROP_RENDERER_VULKAN_DEVICE_POINTER: bytes = "SDL.renderer.vulkan.device".encode()
SDL_PROP_RENDERER_VULKAN_GRAPHICS_QUEUE_FAMILY_INDEX_NUMBER: bytes = "SDL.renderer.vulkan.graphics_queue_family_index".encode()
SDL_PROP_RENDERER_VULKAN_PRESENT_QUEUE_FAMILY_INDEX_NUMBER: bytes = "SDL.renderer.vulkan.present_queue_family_index".encode()
SDL_PROP_RENDERER_VULKAN_SWAPCHAIN_IMAGE_COUNT_NUMBER: bytes = "SDL.renderer.vulkan.swapchain_image_count".encode()
SDL_PROP_RENDERER_GPU_DEVICE_POINTER: bytes = "SDL.renderer.gpu.device".encode()

SDL_GetRenderOutputSize: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderOutputSize", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_BINARY]
SDL_GetCurrentRenderOutputSize: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetCurrentRenderOutputSize", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_BINARY]

SDL_CreateTexture: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateTexture", SDL_POINTER[SDL_Texture], [SDL_POINTER[SDL_Renderer], SDL_PixelFormat, SDL_TextureAccess, ctypes.c_int, ctypes.c_int], SDL_BINARY]
SDL_CreateTextureFromSurface: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateTextureFromSurface", SDL_POINTER[SDL_Texture], [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Surface]], SDL_BINARY]
SDL_CreateTextureWithProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateTextureWithProperties", SDL_POINTER[SDL_Texture], [SDL_POINTER[SDL_Renderer], SDL_PropertiesID], SDL_BINARY]

SDL_PROP_TEXTURE_CREATE_COLORSPACE_NUMBER: bytes = "SDL.texture.create.colorspace".encode()
SDL_PROP_TEXTURE_CREATE_FORMAT_NUMBER: bytes = "SDL.texture.create.format".encode()
SDL_PROP_TEXTURE_CREATE_ACCESS_NUMBER: bytes = "SDL.texture.create.access".encode()
SDL_PROP_TEXTURE_CREATE_WIDTH_NUMBER: bytes = "SDL.texture.create.width".encode()
SDL_PROP_TEXTURE_CREATE_HEIGHT_NUMBER: bytes = "SDL.texture.create.height".encode()
SDL_PROP_TEXTURE_CREATE_SDR_WHITE_POINT_FLOAT: bytes = "SDL.texture.create.SDR_white_point".encode()
SDL_PROP_TEXTURE_CREATE_HDR_HEADROOM_FLOAT: bytes = "SDL.texture.create.HDR_headroom".encode()
SDL_PROP_TEXTURE_CREATE_D3D11_TEXTURE_POINTER: bytes = "SDL.texture.create.d3d11.texture".encode()
SDL_PROP_TEXTURE_CREATE_D3D11_TEXTURE_U_POINTER: bytes = "SDL.texture.create.d3d11.texture_u".encode()
SDL_PROP_TEXTURE_CREATE_D3D11_TEXTURE_V_POINTER: bytes = "SDL.texture.create.d3d11.texture_v".encode()
SDL_PROP_TEXTURE_CREATE_D3D12_TEXTURE_POINTER: bytes = "SDL.texture.create.d3d12.texture".encode()
SDL_PROP_TEXTURE_CREATE_D3D12_TEXTURE_U_POINTER: bytes = "SDL.texture.create.d3d12.texture_u".encode()
SDL_PROP_TEXTURE_CREATE_D3D12_TEXTURE_V_POINTER: bytes = "SDL.texture.create.d3d12.texture_v".encode()
SDL_PROP_TEXTURE_CREATE_METAL_PIXELBUFFER_POINTER: bytes = "SDL.texture.create.metal.pixelbuffer".encode()
SDL_PROP_TEXTURE_CREATE_OPENGL_TEXTURE_NUMBER: bytes = "SDL.texture.create.opengl.texture".encode()
SDL_PROP_TEXTURE_CREATE_OPENGL_TEXTURE_UV_NUMBER: bytes = "SDL.texture.create.opengl.texture_uv".encode()
SDL_PROP_TEXTURE_CREATE_OPENGL_TEXTURE_U_NUMBER: bytes = "SDL.texture.create.opengl.texture_u".encode()
SDL_PROP_TEXTURE_CREATE_OPENGL_TEXTURE_V_NUMBER: bytes = "SDL.texture.create.opengl.texture_v".encode()
SDL_PROP_TEXTURE_CREATE_OPENGLES2_TEXTURE_NUMBER: bytes = "SDL.texture.create.opengles2.texture".encode()
SDL_PROP_TEXTURE_CREATE_OPENGLES2_TEXTURE_UV_NUMBER: bytes = "SDL.texture.create.opengles2.texture_uv".encode()
SDL_PROP_TEXTURE_CREATE_OPENGLES2_TEXTURE_U_NUMBER: bytes = "SDL.texture.create.opengles2.texture_u".encode()
SDL_PROP_TEXTURE_CREATE_OPENGLES2_TEXTURE_V_NUMBER: bytes = "SDL.texture.create.opengles2.texture_v".encode()
SDL_PROP_TEXTURE_CREATE_VULKAN_TEXTURE_NUMBER: bytes = "SDL.texture.create.vulkan.texture".encode()

SDL_GetTextureProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetTextureProperties", SDL_PropertiesID, [SDL_POINTER[SDL_Texture]], SDL_BINARY]

SDL_PROP_TEXTURE_COLORSPACE_NUMBER: bytes = "SDL.texture.colorspace".encode()
SDL_PROP_TEXTURE_FORMAT_NUMBER: bytes = "SDL.texture.format".encode()
SDL_PROP_TEXTURE_ACCESS_NUMBER: bytes = "SDL.texture.access".encode()
SDL_PROP_TEXTURE_WIDTH_NUMBER: bytes = "SDL.texture.width".encode()
SDL_PROP_TEXTURE_HEIGHT_NUMBER: bytes = "SDL.texture.height".encode()
SDL_PROP_TEXTURE_SDR_WHITE_POINT_FLOAT: bytes = "SDL.texture.SDR_white_point".encode()
SDL_PROP_TEXTURE_HDR_HEADROOM_FLOAT: bytes = "SDL.texture.HDR_headroom".encode()
SDL_PROP_TEXTURE_D3D11_TEXTURE_POINTER: bytes = "SDL.texture.d3d11.texture".encode()
SDL_PROP_TEXTURE_D3D11_TEXTURE_U_POINTER: bytes = "SDL.texture.d3d11.texture_u".encode()
SDL_PROP_TEXTURE_D3D11_TEXTURE_V_POINTER: bytes = "SDL.texture.d3d11.texture_v".encode()
SDL_PROP_TEXTURE_D3D12_TEXTURE_POINTER: bytes = "SDL.texture.d3d12.texture".encode()
SDL_PROP_TEXTURE_D3D12_TEXTURE_U_POINTER: bytes = "SDL.texture.d3d12.texture_u".encode()
SDL_PROP_TEXTURE_D3D12_TEXTURE_V_POINTER: bytes = "SDL.texture.d3d12.texture_v".encode()
SDL_PROP_TEXTURE_OPENGL_TEXTURE_NUMBER: bytes = "SDL.texture.opengl.texture".encode()
SDL_PROP_TEXTURE_OPENGL_TEXTURE_UV_NUMBER: bytes = "SDL.texture.opengl.texture_uv".encode()
SDL_PROP_TEXTURE_OPENGL_TEXTURE_U_NUMBER: bytes = "SDL.texture.opengl.texture_u".encode()
SDL_PROP_TEXTURE_OPENGL_TEXTURE_V_NUMBER: bytes = "SDL.texture.opengl.texture_v".encode()
SDL_PROP_TEXTURE_OPENGL_TEXTURE_TARGET_NUMBER: bytes = "SDL.texture.opengl.target".encode()
SDL_PROP_TEXTURE_OPENGL_TEX_W_FLOAT: bytes = "SDL.texture.opengl.tex_w".encode()
SDL_PROP_TEXTURE_OPENGL_TEX_H_FLOAT: bytes = "SDL.texture.opengl.tex_h".encode()
SDL_PROP_TEXTURE_OPENGLES2_TEXTURE_NUMBER: bytes = "SDL.texture.opengles2.texture".encode()
SDL_PROP_TEXTURE_OPENGLES2_TEXTURE_UV_NUMBER: bytes = "SDL.texture.opengles2.texture_uv".encode()
SDL_PROP_TEXTURE_OPENGLES2_TEXTURE_U_NUMBER: bytes = "SDL.texture.opengles2.texture_u".encode()
SDL_PROP_TEXTURE_OPENGLES2_TEXTURE_V_NUMBER: bytes = "SDL.texture.opengles2.texture_v".encode()
SDL_PROP_TEXTURE_OPENGLES2_TEXTURE_TARGET_NUMBER: bytes = "SDL.texture.opengles2.target".encode()
SDL_PROP_TEXTURE_VULKAN_TEXTURE_NUMBER: bytes = "SDL.texture.vulkan.texture".encode()

SDL_GetRendererFromTexture: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRendererFromTexture", SDL_POINTER[SDL_Renderer], [SDL_POINTER[SDL_Texture]], SDL_BINARY]
SDL_GetTextureSize: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetTextureSize", ctypes.c_bool, [SDL_POINTER[SDL_Texture], SDL_POINTER[ctypes.c_float], SDL_POINTER[ctypes.c_float]], SDL_BINARY]

SDL_SetTextureColorMod: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetTextureColorMod", ctypes.c_bool, [SDL_POINTER[SDL_Texture], ctypes.c_uint8, ctypes.c_uint8, ctypes.c_uint8], SDL_BINARY]
SDL_SetTextureColorModFloat: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetTextureColorModFloat", ctypes.c_bool, [SDL_POINTER[SDL_Texture], ctypes.c_float, ctypes.c_float, ctypes.c_float], SDL_BINARY]

SDL_GetTextureColorMod: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetTextureColorMod", ctypes.c_bool, [SDL_POINTER[SDL_Texture], SDL_POINTER[ctypes.c_uint8], SDL_POINTER[ctypes.c_uint8], SDL_POINTER[ctypes.c_uint8]], SDL_BINARY]
SDL_GetTextureColorModFloat: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetTextureColorModFloat", ctypes.c_bool, [SDL_POINTER[SDL_Texture], SDL_POINTER[ctypes.c_float], SDL_POINTER[ctypes.c_float], SDL_POINTER[ctypes.c_float]], SDL_BINARY]

SDL_SetTextureAlphaMod: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetTextureAlphaMod", ctypes.c_bool, [SDL_POINTER[SDL_Texture], ctypes.c_uint8], SDL_BINARY]
SDL_SetTextureAlphaModFloat: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetTextureAlphaModFloat", ctypes.c_bool, [SDL_POINTER[SDL_Texture], ctypes.c_float], SDL_BINARY]

SDL_GetTextureAlphaMod: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetTextureAlphaMod", ctypes.c_bool, [SDL_POINTER[SDL_Texture], SDL_POINTER[ctypes.c_uint8]], SDL_BINARY]
SDL_GetTextureAlphaModFloat: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetTextureAlphaModFloat", ctypes.c_bool, [SDL_POINTER[SDL_Texture], SDL_POINTER[ctypes.c_float]], SDL_BINARY]

SDL_SetTextureBlendMode: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetTextureBlendMode", ctypes.c_bool, [SDL_POINTER[SDL_Texture], SDL_BlendMode], SDL_BINARY]
SDL_GetTextureBlendMode: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetTextureBlendMode", ctypes.c_bool, [SDL_POINTER[SDL_Texture], SDL_POINTER[SDL_BlendMode]], SDL_BINARY]

SDL_SetTextureScaleMode: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetTextureScaleMode", ctypes.c_bool, [SDL_POINTER[SDL_Texture], SDL_ScaleMode], SDL_BINARY]
SDL_GetTextureScaleMode: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetTextureScaleMode", ctypes.c_bool, [SDL_POINTER[SDL_Texture], SDL_POINTER[SDL_ScaleMode]], SDL_BINARY]

SDL_UpdateTexture: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_UpdateTexture", ctypes.c_bool, [SDL_POINTER[SDL_Texture], SDL_POINTER[SDL_Rect], ctypes.c_void_p, ctypes.c_int], SDL_BINARY]
SDL_UpdateYUVTexture: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_UpdateYUVTexture", ctypes.c_bool, [SDL_POINTER[SDL_Texture], SDL_POINTER[SDL_Rect], SDL_POINTER[ctypes.c_uint8], ctypes.c_int, SDL_POINTER[ctypes.c_uint8], ctypes.c_int, SDL_POINTER[ctypes.c_uint8], ctypes.c_int], SDL_BINARY]
SDL_UpdateNVTexture: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_UpdateNVTexture", ctypes.c_bool, [SDL_POINTER[SDL_Texture], SDL_POINTER[SDL_Rect], SDL_POINTER[ctypes.c_uint8], ctypes.c_int, SDL_POINTER[ctypes.c_uint8], ctypes.c_int], SDL_BINARY]

SDL_LockTexture: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LockTexture", ctypes.c_bool, [SDL_POINTER[SDL_Texture], SDL_POINTER[SDL_Rect], SDL_POINTER[ctypes.c_void_p], SDL_POINTER[ctypes.c_int]], SDL_BINARY]
SDL_LockTextureToSurface: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LockTextureToSurface", ctypes.c_bool, [SDL_POINTER[SDL_Texture], SDL_POINTER[SDL_Rect], SDL_POINTER[SDL_POINTER[SDL_Surface]]], SDL_BINARY]
SDL_UnlockTexture: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_UnlockTexture", None, [SDL_POINTER[SDL_Texture]], SDL_BINARY]

SDL_SetRenderTarget: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetRenderTarget", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Texture]], SDL_BINARY]
SDL_GetRenderTarget: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderTarget", SDL_POINTER[SDL_Texture], [SDL_POINTER[SDL_Renderer]], SDL_BINARY]

SDL_SetRenderLogicalPresentation: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetRenderLogicalPresentation", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], ctypes.c_int, ctypes.c_int, SDL_RendererLogicalPresentation], SDL_BINARY]
SDL_GetRenderLogicalPresentation: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderLogicalPresentation", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int], SDL_POINTER[SDL_RendererLogicalPresentation]], SDL_BINARY]
SDL_GetRenderLogicalPresentationRect: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderLogicalPresentationRect", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_FRect]], SDL_BINARY]

SDL_RenderCoordinatesFromWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderCoordinatesFromWindow", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], ctypes.c_float, ctypes.c_float, SDL_POINTER[ctypes.c_float], SDL_POINTER[ctypes.c_float]], SDL_BINARY]
SDL_RenderCoordinatesToWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderCoordinatesToWindow", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], ctypes.c_float, ctypes.c_float, SDL_POINTER[ctypes.c_float], SDL_POINTER[ctypes.c_float]], SDL_BINARY]

SDL_ConvertEventToRenderCoordinates: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ConvertEventToRenderCoordinates", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Event]], SDL_BINARY]

SDL_SetRenderViewport: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetRenderViewport", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Rect]], SDL_BINARY]
SDL_GetRenderViewport: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderViewport", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Rect]], SDL_BINARY]

SDL_RenderViewportSet: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderViewportSet", ctypes.c_bool, [SDL_POINTER[SDL_Renderer]], SDL_BINARY]
SDL_GetRenderSafeArea: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderSafeArea", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Rect]], SDL_BINARY]

SDL_SetRenderClipRect: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetRenderClipRect", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Rect]], SDL_BINARY]
SDL_GetRenderClipRect: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderClipRect", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Rect]], SDL_BINARY]

SDL_RenderClipEnabled: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderClipEnabled", ctypes.c_bool, [SDL_POINTER[SDL_Renderer]], SDL_BINARY]

SDL_SetRenderScale: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetRenderScale", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], ctypes.c_float, ctypes.c_float], SDL_BINARY]
SDL_GetRenderScale: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderScale", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[ctypes.c_float], SDL_POINTER[ctypes.c_float]], SDL_BINARY]

SDL_SetRenderDrawColor: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetRenderDrawColor", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], ctypes.c_uint8, ctypes.c_uint8, ctypes.c_uint8, ctypes.c_uint8], SDL_BINARY]
SDL_SetRenderDrawColorFloat: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetRenderDrawColorFloat", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], ctypes.c_float, ctypes.c_float, ctypes.c_float, ctypes.c_float], SDL_BINARY]

SDL_GetRenderDrawColor: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderDrawColor", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[ctypes.c_uint8], SDL_POINTER[ctypes.c_uint8], SDL_POINTER[ctypes.c_uint8], SDL_POINTER[ctypes.c_uint8]], SDL_BINARY]
SDL_GetRenderDrawColorFloat: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderDrawColorFloat", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[ctypes.c_float], SDL_POINTER[ctypes.c_float], SDL_POINTER[ctypes.c_float], SDL_POINTER[ctypes.c_float]], SDL_BINARY]

SDL_SetRenderColorScale: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetRenderColorScale", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], ctypes.c_float], SDL_BINARY]
SDL_GetRenderColorScale: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderColorScale", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[ctypes.c_float]], SDL_BINARY]

SDL_SetRenderDrawBlendMode: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetRenderDrawBlendMode", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_BlendMode], SDL_BINARY]
SDL_GetRenderDrawBlendMode: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderDrawBlendMode", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_BlendMode]], SDL_BINARY]

SDL_RenderClear: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderClear", ctypes.c_bool, [SDL_POINTER[SDL_Renderer]], SDL_BINARY]

SDL_RenderPoint: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderPoint", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], ctypes.c_float, ctypes.c_float], SDL_BINARY]
SDL_RenderPoints: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderPoints", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_FPoint], ctypes.c_int], SDL_BINARY]

SDL_RenderLine: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderLine", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], ctypes.c_float, ctypes.c_float, ctypes.c_float, ctypes.c_float], SDL_BINARY]
SDL_RenderLines: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderLines", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_FPoint], ctypes.c_int], SDL_BINARY]

SDL_RenderRect: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderRect", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_FRect]], SDL_BINARY]
SDL_RenderRects: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderRects", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_FRect], ctypes.c_int], SDL_BINARY]

SDL_RenderFillRect: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderFillRect", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_FRect]], SDL_BINARY]
SDL_RenderFillRects: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderFillRects", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_FRect], ctypes.c_int], SDL_BINARY]

SDL_RenderTexture: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderTexture", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Texture], SDL_POINTER[SDL_FRect], SDL_POINTER[SDL_FRect]], SDL_BINARY]
SDL_RenderTextureRotated: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderTextureRotated", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Texture], SDL_POINTER[SDL_FRect], SDL_POINTER[SDL_FRect], ctypes.c_double, SDL_POINTER[SDL_FPoint], SDL_FlipMode], SDL_BINARY]
SDL_RenderTextureAffine: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderTextureAffine", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Texture], SDL_POINTER[SDL_FRect], SDL_POINTER[SDL_FPoint], SDL_POINTER[SDL_FPoint], SDL_POINTER[SDL_FPoint]], SDL_BINARY]
SDL_RenderTextureTiled: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderTextureTiled", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Texture], SDL_POINTER[SDL_FRect], ctypes.c_float, SDL_POINTER[SDL_FRect]], SDL_BINARY]
SDL_RenderTexture9Grid: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderTexture9Grid", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Texture], SDL_POINTER[SDL_FRect], ctypes.c_float, ctypes.c_float, ctypes.c_float, ctypes.c_float, ctypes.c_float, SDL_POINTER[SDL_FRect]], SDL_BINARY]

SDL_RenderGeometry: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderGeometry", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Texture], SDL_POINTER[SDL_Vertex], ctypes.c_int, SDL_POINTER[ctypes.c_int], ctypes.c_int], SDL_BINARY]
SDL_RenderGeometryRaw: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderGeometryRaw", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Texture], SDL_POINTER[ctypes.c_float], ctypes.c_int, SDL_POINTER[SDL_FColor], ctypes.c_int, SDL_POINTER[ctypes.c_float], ctypes.c_int, ctypes.c_int, ctypes.c_void_p, ctypes.c_int, ctypes.c_int], SDL_BINARY]

SDL_RenderReadPixels: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderReadPixels", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_Rect]], SDL_BINARY]
SDL_RenderPresent: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderPresent", ctypes.c_bool, [SDL_POINTER[SDL_Renderer]], SDL_BINARY]

SDL_DestroyTexture: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_DestroyTexture", None, [SDL_POINTER[SDL_Texture]], SDL_BINARY]
SDL_DestroyRenderer: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_DestroyRenderer", None, [SDL_POINTER[SDL_Renderer]], SDL_BINARY]
SDL_FlushRenderer: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_FlushRenderer", ctypes.c_bool, [SDL_POINTER[SDL_Renderer]], SDL_BINARY]

SDL_GetRenderMetalLayer: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderMetalLayer", ctypes.c_void_p, [SDL_POINTER[SDL_Renderer]], SDL_BINARY]
SDL_GetRenderMetalCommandEncoder: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderMetalCommandEncoder", ctypes.c_void_p, [SDL_POINTER[SDL_Renderer]], SDL_BINARY]

SDL_AddVulkanRenderSemaphores: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_AddVulkanRenderSemaphores", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], ctypes.c_uint32, ctypes.c_int64, ctypes.c_int64], SDL_BINARY]

SDL_SetRenderVSync: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetRenderVSync", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], ctypes.c_int], SDL_BINARY]

SDL_RENDERER_VSYNC_DISABLED, SDL_RENDERER_VSYNC_ADAPTIVE = 0, -1

SDL_GetRenderVSync: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetRenderVSync", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], SDL_POINTER[ctypes.c_int]], SDL_BINARY]

SDL_DEBUG_TEXT_FONT_CHARACTER_SIZE: int = 8

SDL_RenderDebugText: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderDebugText", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], ctypes.c_float, ctypes.c_float, ctypes.c_char_p], SDL_BINARY]
SDL_RenderDebugTextFormat: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RenderDebugTextFormat", ctypes.c_bool, [SDL_POINTER[SDL_Renderer], ctypes.c_float, ctypes.c_float, ctypes.c_char_p, ...], SDL_BINARY]



from .__init__ import ctypes, typing, abc, SDL_POINTER, SDL_VA_LIST, \
    SDL_FUNC_TYPE, SDL_FUNC, SDL_TYPE, SDL_BINARY, SDL_ENUM

SDL_LogCategory: typing.TypeAlias = SDL_TYPE["SDL_LogCategory", SDL_ENUM]

SDL_LOG_CATEGORY_APPLICATION, SDL_LOG_CATEGORY_ERROR, SDL_LOG_CATEGORY_ASSERT, SDL_LOG_CATEGORY_SYSTEM, SDL_LOG_CATEGORY_AUDIO, \
    SDL_LOG_CATEGORY_VIDEO, SDL_LOG_CATEGORY_RENDER, SDL_LOG_CATEGORY_INPUT, SDL_LOG_CATEGORY_TEST, SDL_LOG_CATEGORY_GPU, SDL_LOG_CATEGORY_RESERVED1, \
        SDL_LOG_CATEGORY_RESERVED2, SDL_LOG_CATEGORY_RESERVED3, SDL_LOG_CATEGORY_RESERVED4, SDL_LOG_CATEGORY_RESERVED5, SDL_LOG_CATEGORY_RESERVED6, SDL_LOG_CATEGORY_RESERVED7, \
            SDL_LOG_CATEGORY_RESERVED8, SDL_LOG_CATEGORY_RESERVED9, SDL_LOG_CATEGORY_RESERVED10, SDL_LOG_CATEGORY_CUSTOM = range(21)

SDL_LogPriority: typing.TypeAlias = SDL_TYPE["SDL_LogPriority", SDL_ENUM]

SDL_LOG_PRIORITY_INVALID, SDL_LOG_PRIORITY_TRACE, SDL_LOG_PRIORITY_VERBOSE, SDL_LOG_PRIORITY_DEBUG, SDL_LOG_PRIORITY_INFO, \
    SDL_LOG_PRIORITY_WARN, SDL_LOG_PRIORITY_ERROR, SDL_LOG_PRIORITY_CRITICAL, SDL_LOG_PRIORITY_COUNT = range(9)

SDL_SetLogPriorities: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetLogPriorities", None, [SDL_LogPriority], SDL_BINARY]
SDL_SetLogPriority: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetLogPriority", None, [ctypes.c_int, SDL_LogPriority], SDL_BINARY]
SDL_GetLogPriority: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetLogPriority", SDL_LogPriority, [ctypes.c_int], SDL_BINARY]
SDL_ResetLogPriorities: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ResetLogPriorities", None, [], SDL_BINARY]
SDL_SetLogPriorityPrefix: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetLogPriorityPrefix", ctypes.c_bool, [SDL_LogPriority, ctypes.c_char_p], SDL_BINARY]

SDL_Log: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_Log", None, [ctypes.c_char_p, ...], SDL_BINARY]
SDL_LogTrace: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LogTrace", None, [ctypes.c_int, ctypes.c_char_p, ...], SDL_BINARY]
SDL_LogVerbose: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LogVerbose", None, [ctypes.c_int, ctypes.c_char_p, ...], SDL_BINARY]
SDL_LogDebug: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LogDebug", None, [ctypes.c_int, ctypes.c_char_p, ...], SDL_BINARY]
SDL_LogInfo: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LogInfo", None, [ctypes.c_int, ctypes.c_char_p, ...], SDL_BINARY]
SDL_LogWarn: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LogWarn", None, [ctypes.c_int, ctypes.c_char_p, ...], SDL_BINARY]
SDL_LogError: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LogError", None, [ctypes.c_int, ctypes.c_char_p, ...], SDL_BINARY]
SDL_LogCritical: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LogCritical", None, [ctypes.c_int, ctypes.c_char_p, ...], SDL_BINARY]
SDL_LogMessage: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LogMessage", None, [ctypes.c_int, SDL_LogPriority, ctypes.c_char_p, ...], SDL_BINARY]
SDL_LogMessageV: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LogMessageV", None, [ctypes.c_int, SDL_LogPriority, ctypes.c_char_p, SDL_VA_LIST], SDL_BINARY]

SDL_LogOutputFunction: typing.TypeAlias = SDL_FUNC_TYPE["SDL_LogOutputFunction", None, [ctypes.c_void_p, ctypes.c_int, SDL_LogPriority, ctypes.c_char_p]]

SDL_GetDefaultLogOutputFunction: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetDefaultLogOutputFunction", SDL_LogOutputFunction, [], SDL_BINARY]
SDL_GetLogOutputFunction: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetLogOutputFunction", None, [SDL_POINTER[SDL_LogOutputFunction], SDL_POINTER[ctypes.c_void_p]], SDL_BINARY]
SDL_SetLogOutputFunction: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetLogOutputFunction", None, [SDL_LogOutputFunction, ctypes.c_void_p], SDL_BINARY]


from .__init__ import ctypes, typing, abc, SDL_PLATFORM_SPECIFIC, \
    SDL_POINTER, SDL_FUNC, SDL_TYPE, SDL_TTF_BINARY, SDL_ENUM

from .SDL_pixels import SDL_Color
from .SDL_surface import SDL_Surface
from .SDL_rect import SDL_Rect, SDL_FPoint
from .SDL_gpu import SDL_GPUDevice, SDL_GPUTexture
from .SDL_properties import SDL_PropertiesID
from .SDL_version import SDL_VERSIONNUM
from .SDL_iostream import SDL_IOStream
from .SDL_render import SDL_Renderer

SDL_TTF_MAJOR_VERSION, SDL_TTF_MINOR_VERSION, SDL_TTF_MICRO_VERSION = 3, 2, 2
SDL_TTF_VERSION: int = SDL_VERSIONNUM(SDL_TTF_MAJOR_VERSION, SDL_TTF_MINOR_VERSION, SDL_TTF_MICRO_VERSION)

SDL_TTF_VERSION_ATLEAST: abc.Callable[[int, int, int], bool] = lambda x, y, z: \
    (SDL_TTF_MAJOR_VERSION >= x) and (SDL_TTF_MAJOR_VERSION > x or SDL_TTF_MINOR_VERSION >= y) and \
        (SDL_TTF_MAJOR_VERSION > x or SDL_TTF_MINOR_VERSION > y or SDL_TTF_MICRO_VERSION >= z)

TTF_Version: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_Version", ctypes.c_int, [], SDL_TTF_BINARY]
TTF_GetFreeTypeVersion: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFreeTypeVersion", None, [SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_TTF_BINARY]
TTF_GetHarfBuzzVersion: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetHarfBuzzVersion", None, [SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_TTF_BINARY]

class TTF_Font(ctypes.c_void_p):
    ...

TTF_Init: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_Init", ctypes.c_bool, [], SDL_TTF_BINARY]
TTF_OpenFont: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_OpenFont", SDL_POINTER[TTF_Font], [ctypes.c_char_p, ctypes.c_float], SDL_TTF_BINARY]
TTF_OpenFontIO: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_OpenFontIO", SDL_POINTER[TTF_Font], [SDL_POINTER[SDL_IOStream], ctypes.c_bool, ctypes.c_float], SDL_TTF_BINARY]
TTF_OpenFontWithProperties: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_OpenFontWithProperties", SDL_POINTER[TTF_Font], [SDL_PropertiesID], SDL_TTF_BINARY]

TTF_PROP_FONT_CREATE_FILENAME_STRING: bytes = "SDL_ttf.font.create.filename".encode()
TTF_PROP_FONT_CREATE_IOSTREAM_POINTER: bytes = "SDL_ttf.font.create.iostream".encode()
TTF_PROP_FONT_CREATE_IOSTREAM_OFFSET_NUMBER: bytes = "SDL_ttf.font.create.iostream.offset".encode()
TTF_PROP_FONT_CREATE_IOSTREAM_AUTOCLOSE_BOOLEAN: bytes = "SDL_ttf.font.create.iostream.autoclose".encode()
TTF_PROP_FONT_CREATE_SIZE_FLOAT: bytes = "SDL_ttf.font.create.size".encode()
TTF_PROP_FONT_CREATE_FACE_NUMBER: bytes = "SDL_ttf.font.create.face".encode()
TTF_PROP_FONT_CREATE_HORIZONTAL_DPI_NUMBER: bytes = "SDL_ttf.font.create.hdpi".encode()
TTF_PROP_FONT_CREATE_VERTICAL_DPI_NUMBER: bytes = "SDL_ttf.font.create.vdpi".encode()
TTF_PROP_FONT_CREATE_EXISTING_FONT: bytes = "SDL_ttf.font.create.existing_font".encode()

TTF_CopyFont: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_CopyFont", SDL_POINTER[TTF_Font], [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_GetFontProperties: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontProperties", SDL_PropertiesID, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_PROP_FONT_OUTLINE_LINE_CAP_NUMBER: bytes = "SDL_ttf.font.outline.line_cap".encode()
TTF_PROP_FONT_OUTLINE_LINE_JOIN_NUMBER: bytes = "SDL_ttf.font.outline.line_join".encode()
TTF_PROP_FONT_OUTLINE_MITER_LIMIT_NUMBER: bytes = "SDL_ttf.font.outline.miter_limit".encode()

TTF_GetFontGeneration: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontGeneration", ctypes.c_uint32, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_AddFallbackFont: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_AddFallbackFont", ctypes.c_bool, [SDL_POINTER[TTF_Font], SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]
TTF_RemoveFallbackFont: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_RemoveFallbackFont", None, [SDL_POINTER[TTF_Font], SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]
TTF_ClearFallbackFonts: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_ClearFallbackFonts", None, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_SetFontSize: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetFontSize", ctypes.c_bool, [SDL_POINTER[TTF_Font], ctypes.c_float], SDL_TTF_BINARY]
TTF_SetFontSizeDPI: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetFontSizeDPI", ctypes.c_bool, [SDL_POINTER[TTF_Font], ctypes.c_float, ctypes.c_int, ctypes.c_int], SDL_TTF_BINARY]

TTF_GetFontSize: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontSize", ctypes.c_float, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]
TTF_GetFontDPI: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontDPI", ctypes.c_bool, [SDL_POINTER[TTF_Font], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_TTF_BINARY]

TTF_FontStyleFlags: typing.TypeAlias = SDL_TYPE["TTF_FontStyleFlags", ctypes.c_uint32]

TTF_STYLE_NORMAL, TTF_STYLE_BOLD, TTF_STYLE_ITALIC, TTF_STYLE_UNDERLINE, \
     TTF_STYLE_STRIKETHROUGH = 0x00, 0x01, 0x02, 0x04, 0x08

TTF_SetFontStyle: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetFontStyle", None, [SDL_POINTER[TTF_Font], TTF_FontStyleFlags], SDL_TTF_BINARY]
TTF_GetFontStyle: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontStyle", TTF_FontStyleFlags, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]
TTF_SetFontOutline: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetFontOutline", ctypes.c_bool, [SDL_POINTER[TTF_Font], ctypes.c_int], SDL_TTF_BINARY]
TTF_GetFontOutline: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontOutline", ctypes.c_int, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_HintingFlags: typing.TypeAlias = SDL_TYPE["TTF_HintingFlags", SDL_ENUM]

TTF_HINTING_INVALID, TTF_HINTING_NORMAL, TTF_HINTING_LIGHT, TTF_HINTING_MONO, TTF_HINTING_NONE, TTF_HINTING_LIGHT_SUBPIXEL = range(-1, 5)

TTF_SetFontHinting: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetFontHinting", None, [SDL_POINTER[TTF_Font], TTF_HintingFlags], SDL_TTF_BINARY]
TTF_GetNumFontFaces: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetNumFontFaces", ctypes.c_int, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]
TTF_GetFontHinting: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontHinting", TTF_HintingFlags, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_SetFontSDF: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetFontSDF", ctypes.c_bool, [SDL_POINTER[TTF_Font], ctypes.c_bool], SDL_TTF_BINARY]
TTF_GetFontSDF: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontSDF", ctypes.c_bool, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_GetFontWeight: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontWeight", ctypes.c_int, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_FONT_WEIGHT_THIN, TTF_FONT_WEIGHT_EXTRA_LIGHT, TTF_FONT_WEIGHT_LIGHT, TTF_FONT_WEIGHT_NORMAL, TTF_FONT_WEIGHT_MEDIUM, TTF_FONT_WEIGHT_SEMI_BOLD, \
    TTF_FONT_WEIGHT_BOLD, TTF_FONT_WEIGHT_EXTRA_BOLD, TTF_FONT_WEIGHT_BLACK, TTF_FONT_WEIGHT_EXTRA_BLACK = *range(100, 1000, 100), 950

TTF_HorizontalAlignment: typing.TypeAlias = SDL_TYPE["TTF_HorizontalAlignment", SDL_ENUM]

TTF_HORIZONTAL_ALIGN_INVALID, TTF_HORIZONTAL_ALIGN_LEFT, TTF_HORIZONTAL_ALIGN_CENTER, TTF_HORIZONTAL_ALIGN_RIGHT = range(-1, 3)

TTF_SetFontWrapAlignment: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetFontWrapAlignment", None, [SDL_POINTER[TTF_Font], TTF_HorizontalAlignment], SDL_TTF_BINARY]
TTF_GetFontWrapAlignment: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontWrapAlignment", TTF_HorizontalAlignment, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_GetFontHeight: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontHeight", ctypes.c_int, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]
TTF_GetFontAscent: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontAscent", ctypes.c_int, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]
TTF_GetFontDescent: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontDescent", ctypes.c_int, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_SetFontLineSkip: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetFontLineSkip", None, [SDL_POINTER[TTF_Font], ctypes.c_int], SDL_TTF_BINARY]
TTF_GetFontLineSkip: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontLineSkip", ctypes.c_int, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_SetFontKerning: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetFontKerning", None, [SDL_POINTER[TTF_Font], ctypes.c_bool], SDL_TTF_BINARY]
TTF_GetFontKerning: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontKerning", ctypes.c_bool, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_FontIsFixedWidth: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_FontIsFixedWidth", ctypes.c_bool, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]
TTF_FontIsScalable: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_FontIsScalable", ctypes.c_bool, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_GetFontFamilyName: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontFamilyName", ctypes.c_char_p, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]
TTF_GetFontStyleName: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontStyleName", ctypes.c_char_p, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_Direction: typing.TypeAlias = SDL_TYPE["TTF_Direction", SDL_ENUM]

TTF_DIRECTION_INVALID, (TTF_DIRECTION_LTR, TTF_DIRECTION_RTL, TTF_DIRECTION_TTB, TTF_DIRECTION_BTT) = 0, range(4, 8)

TTF_SetFontDirection: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetFontDirection", ctypes.c_bool, [SDL_POINTER[TTF_Font], TTF_Direction], SDL_TTF_BINARY]
TTF_GetFontDirection: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontDirection", TTF_Direction, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_StringToTag: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_StringToTag", ctypes.c_uint32, [ctypes.c_char_p], SDL_TTF_BINARY]
TTF_TagToString: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_TagToString", None, [ctypes.c_uint32, ctypes.c_char_p, ctypes.c_size_t], SDL_TTF_BINARY]

TTF_SetFontScript: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetFontScript", ctypes.c_bool, [SDL_POINTER[TTF_Font], ctypes.c_uint32], SDL_TTF_BINARY]
TTF_GetFontScript: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetFontScript", ctypes.c_uint32, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]

TTF_GetGlyphScript: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetGlyphScript", ctypes.c_uint32, [ctypes.c_uint32], SDL_TTF_BINARY]

TTF_SetFontLanguage: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetFontLanguage", ctypes.c_bool, [SDL_POINTER[TTF_Font], ctypes.c_char_p], SDL_TTF_BINARY]
TTF_FontHasGlyph: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_FontHasGlyph", ctypes.c_bool, [SDL_POINTER[TTF_Font], ctypes.c_uint32], SDL_TTF_BINARY]

TTF_ImageType: typing.TypeAlias = SDL_TYPE["TTF_ImageType", SDL_ENUM]

TTF_IMAGE_INVALID, TTF_IMAGE_ALPHA, TTF_IMAGE_COLOR, TTF_IMAGE_SDF = range(4)

TTF_GetGlyphImage: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetGlyphImage", SDL_POINTER[SDL_Surface], [SDL_POINTER[TTF_Font], ctypes.c_uint32, SDL_POINTER[TTF_ImageType]], SDL_TTF_BINARY]
TTF_GetGlyphImageForIndex: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetGlyphImageForIndex", SDL_POINTER[SDL_Surface], [SDL_POINTER[TTF_Font], ctypes.c_uint32, SDL_POINTER[TTF_ImageType]], SDL_TTF_BINARY]
TTF_GetGlyphMetrics: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetGlyphMetrics", ctypes.c_bool, [SDL_POINTER[TTF_Font], ctypes.c_uint32, SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_TTF_BINARY]
TTF_GetGlyphKerning: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetGlyphKerning", ctypes.c_bool, [SDL_POINTER[TTF_Font], ctypes.c_uint32, ctypes.c_uint32, SDL_POINTER[ctypes.c_int]], SDL_TTF_BINARY]

TTF_GetStringSize: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetStringSize", ctypes.c_bool, [SDL_POINTER[TTF_Font], ctypes.c_char_p, ctypes.c_size_t, SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_TTF_BINARY]
TTF_GetStringSizeWrapped: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetStringSizeWrapped", ctypes.c_bool, [SDL_POINTER[TTF_Font], ctypes.c_char_p, ctypes.c_size_t, ctypes.c_int, SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_TTF_BINARY]

TTF_MeasureString: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_MeasureString", ctypes.c_bool, [SDL_POINTER[TTF_Font], ctypes.c_char_p, ctypes.c_size_t, ctypes.c_int, SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_size_t]], SDL_TTF_BINARY]

TTF_RenderText_Solid: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_RenderText_Solid", SDL_POINTER[SDL_Surface], [SDL_POINTER[TTF_Font], ctypes.c_char_p, ctypes.c_size_t, SDL_Color], SDL_TTF_BINARY]
TTF_RenderText_Solid_Wrapped: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_RenderText_Solid_Wrapped", SDL_POINTER[SDL_Surface], [SDL_POINTER[TTF_Font], ctypes.c_char_p, ctypes.c_size_t, SDL_Color, ctypes.c_int], SDL_TTF_BINARY]
TTF_RenderGlyph_Solid: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_RenderGlyph_Solid", SDL_POINTER[SDL_Surface], [SDL_POINTER[TTF_Font], ctypes.c_uint32, SDL_Color], SDL_TTF_BINARY]

TTF_RenderText_Shaded: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_RenderText_Shaded", SDL_POINTER[SDL_Surface], [SDL_POINTER[TTF_Font], ctypes.c_char_p, ctypes.c_size_t, SDL_Color, SDL_Color], SDL_TTF_BINARY]
TTF_RenderText_Shaded_Wrapped: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_RenderText_Shaded_Wrapped", SDL_POINTER[SDL_Surface], [SDL_POINTER[TTF_Font], ctypes.c_char_p, ctypes.c_size_t, SDL_Color, SDL_Color, ctypes.c_int], SDL_TTF_BINARY]
TTF_RenderGlyph_Shaded: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_RenderGlyph_Shaded", SDL_POINTER[SDL_Surface], [SDL_POINTER[TTF_Font], ctypes.c_uint32, SDL_Color, SDL_Color], SDL_TTF_BINARY]

TTF_RenderText_Blended: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_RenderText_Blended", SDL_POINTER[SDL_Surface], [SDL_POINTER[TTF_Font], ctypes.c_char_p, ctypes.c_size_t, SDL_Color], SDL_TTF_BINARY]
TTF_RenderText_Blended_Wrapped: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_RenderText_Blended_Wrapped", SDL_POINTER[SDL_Surface], [SDL_POINTER[TTF_Font], ctypes.c_char_p, ctypes.c_size_t, SDL_Color, ctypes.c_int], SDL_TTF_BINARY]
TTF_RenderGlyph_Blended: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_RenderGlyph_Blended", SDL_POINTER[SDL_Surface], [SDL_POINTER[TTF_Font], ctypes.c_uint32, SDL_Color], SDL_TTF_BINARY]

TTF_RenderText_LCD: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_RenderText_LCD", SDL_POINTER[SDL_Surface], [SDL_POINTER[TTF_Font], ctypes.c_char_p, ctypes.c_size_t, SDL_Color, SDL_Color], SDL_TTF_BINARY]
TTF_RenderText_LCD_Wrapped: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_RenderText_LCD_Wrapped", SDL_POINTER[SDL_Surface], [SDL_POINTER[TTF_Font], ctypes.c_char_p, ctypes.c_size_t, SDL_Color, SDL_Color, ctypes.c_int], SDL_TTF_BINARY]
TTF_RenderGlyph_LCD: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_RenderGlyph_LCD", SDL_POINTER[SDL_Surface], [SDL_POINTER[TTF_Font], ctypes.c_uint32, SDL_Color, SDL_Color], SDL_TTF_BINARY]

class TTF_TextEngine(ctypes.c_void_p):
    ...

class TTF_TextData(ctypes.c_void_p):
    ...

class TTF_Text(ctypes.Structure):
    _fields_ = [
        ("text", ctypes.c_char_p),
        ("num_lines", ctypes.c_int),
        ("refcount", ctypes.c_int),
        ("internal", SDL_POINTER[TTF_TextData])
    ]

TTF_CreateSurfaceTextEngine: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_CreateSurfaceTextEngine", SDL_POINTER[TTF_TextEngine], [], SDL_TTF_BINARY]
TTF_DrawSurfaceText: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_DrawSurfaceText", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_int, ctypes.c_int, SDL_POINTER[SDL_Surface]], SDL_TTF_BINARY]
TTF_DestroySurfaceTextEngine: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_DestroySurfaceTextEngine", None, [SDL_POINTER[TTF_TextEngine]], SDL_TTF_BINARY]

TTF_CreateRendererTextEngine: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_CreateRendererTextEngine", SDL_POINTER[TTF_TextEngine], [SDL_POINTER[SDL_Renderer]], SDL_TTF_BINARY]
TTF_CreateRendererTextEngineWithProperties: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_CreateRendererTextEngineWithProperties", SDL_POINTER[TTF_TextEngine], [SDL_PropertiesID], SDL_TTF_BINARY]

TTF_PROP_RENDERER_TEXT_ENGINE_RENDERER: bytes = "SDL_ttf.renderer_text_engine.create.renderer".encode()
TTF_PROP_RENDERER_TEXT_ENGINE_ATLAS_TEXTURE_SIZE: bytes = "SDL_ttf.renderer_text_engine.create.atlas_texture_size".encode()

TTF_DrawRendererText: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_DrawRendererText", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_float, ctypes.c_float], SDL_TTF_BINARY]
TTF_DestroyRendererTextEngine: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_DestroyRendererTextEngine", None, [SDL_POINTER[TTF_TextEngine]], SDL_TTF_BINARY]

TTF_CreateGPUTextEngine: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_CreateGPUTextEngine", SDL_POINTER[TTF_TextEngine], [SDL_POINTER[SDL_GPUDevice]], SDL_TTF_BINARY]
TTF_CreateGPUTextEngineWithProperties: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_CreateGPUTextEngineWithProperties", SDL_POINTER[TTF_TextEngine], [SDL_PropertiesID], SDL_TTF_BINARY]

TTF_PROP_GPU_TEXT_ENGINE_DEVICE: bytes = "SDL_ttf.gpu_text_engine.create.device".encode()
TTF_PROP_GPU_TEXT_ENGINE_ATLAS_TEXTURE_SIZE: bytes = "SDL_ttf.gpu_text_engine.create.atlas_texture_size".encode()

class TTF_GPUAtlasDrawSequence(ctypes.c_void_p):
    ...

class TTF_GPUAtlasDrawSequence(ctypes.Structure):
    _fields_ = [
        ("atlas_texture", SDL_POINTER[SDL_GPUTexture]),
        ("xy", SDL_POINTER[SDL_FPoint]),
        ("uv", SDL_POINTER[SDL_FPoint]),
        ("num_vertices", ctypes.c_int),
        ("indices", SDL_POINTER[ctypes.c_int]),
        ("num_indices", ctypes.c_int),
        ("image_type", TTF_ImageType),
        ("next", SDL_POINTER[TTF_GPUAtlasDrawSequence])
    ]

TTF_GetGPUTextDrawData: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetGPUTextDrawData", SDL_POINTER[TTF_GPUAtlasDrawSequence], [SDL_POINTER[TTF_Text]], SDL_TTF_BINARY]
TTF_DestroyGPUTextEngine: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_DestroyGPUTextEngine", None, [SDL_POINTER[TTF_TextEngine]], SDL_TTF_BINARY]

TTF_GPUTextEngineWinding: typing.TypeAlias = SDL_TYPE["TTF_GPUTextEngineWinding", SDL_ENUM]

TTF_GPU_TEXTENGINE_WINDING_INVALID, TTF_GPU_TEXTENGINE_WINDING_CLOCKWISE, TTF_GPU_TEXTENGINE_WINDING_COUNTER_CLOCKWISE = range(-1, 2)

TTF_SetGPUTextEngineWinding: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetGPUTextEngineWinding", None, [SDL_POINTER[TTF_TextEngine], TTF_GPUTextEngineWinding], SDL_TTF_BINARY]
TTF_GetGPUTextEngineWinding: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetGPUTextEngineWinding", TTF_GPUTextEngineWinding, [SDL_POINTER[TTF_TextEngine]], SDL_TTF_BINARY]

TTF_CreateText: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_CreateText", SDL_POINTER[TTF_Text], [SDL_POINTER[TTF_TextEngine], SDL_POINTER[TTF_Font], ctypes.c_char_p, ctypes.c_size_t], SDL_TTF_BINARY]

if SDL_PLATFORM_SPECIFIC(system = ["Windows"]):
    TTF_GetTextProperties: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetTextProperties", SDL_PropertiesID, [SDL_POINTER[TTF_Text]], SDL_TTF_BINARY]

TTF_SetTextEngine: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetTextEngine", ctypes.c_bool, [SDL_POINTER[TTF_Text], SDL_POINTER[TTF_TextEngine]], SDL_TTF_BINARY]
TTF_GetTextEngine: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetTextEngine", SDL_POINTER[TTF_TextEngine], [SDL_POINTER[TTF_Text]], SDL_TTF_BINARY]

TTF_SetTextFont: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetTextFont", ctypes.c_bool, [SDL_POINTER[TTF_Text], SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]
TTF_GetTextFont: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetTextFont", SDL_POINTER[TTF_Font], [SDL_POINTER[TTF_Text]], SDL_TTF_BINARY]

TTF_SetTextDirection: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetTextDirection", ctypes.c_bool, [SDL_POINTER[TTF_Text], TTF_Direction], SDL_TTF_BINARY]
TTF_GetTextDirection: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetTextDirection", TTF_Direction, [SDL_POINTER[TTF_Text]], SDL_TTF_BINARY]

TTF_SetTextScript: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetTextScript", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_uint32], SDL_TTF_BINARY]
TTF_GetTextScript: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetTextScript", ctypes.c_uint32, [SDL_POINTER[TTF_Text]], SDL_TTF_BINARY]

TTF_SetTextColor: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetTextColor", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_uint8, ctypes.c_uint8, ctypes.c_uint8, ctypes.c_uint8], SDL_TTF_BINARY]
TTF_SetTextColorFloat: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetTextColorFloat", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_float, ctypes.c_float, ctypes.c_float, ctypes.c_float], SDL_TTF_BINARY]

TTF_GetTextColor: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetTextColor", ctypes.c_bool, [SDL_POINTER[TTF_Text], SDL_POINTER[ctypes.c_uint8], SDL_POINTER[ctypes.c_uint8], SDL_POINTER[ctypes.c_uint8], SDL_POINTER[ctypes.c_uint8]], SDL_TTF_BINARY]
TTF_GetTextColorFloat: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetTextColorFloat", ctypes.c_bool, [SDL_POINTER[TTF_Text], SDL_POINTER[ctypes.c_float], SDL_POINTER[ctypes.c_float], SDL_POINTER[ctypes.c_float], SDL_POINTER[ctypes.c_float]], SDL_TTF_BINARY]

TTF_SetTextPosition: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetTextPosition", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_int, ctypes.c_int], SDL_TTF_BINARY]
TTF_GetTextPosition: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetTextPosition", ctypes.c_bool, [SDL_POINTER[TTF_Text], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_TTF_BINARY]

TTF_SetTextWrapWidth: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetTextWrapWidth", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_int], SDL_TTF_BINARY]
TTF_GetTextWrapWidth: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetTextWrapWidth", ctypes.c_bool, [SDL_POINTER[TTF_Text], SDL_POINTER[ctypes.c_int]], SDL_TTF_BINARY]

TTF_SetTextWrapWhitespaceVisible: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetTextWrapWhitespaceVisible", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_bool], SDL_TTF_BINARY]
TTF_TextWrapWhitespaceVisible: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_TextWrapWhitespaceVisible", ctypes.c_bool, [SDL_POINTER[TTF_Text]], SDL_TTF_BINARY]

TTF_SetTextString: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_SetTextString", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_char_p, ctypes.c_size_t], SDL_TTF_BINARY]
TTF_InsertTextString: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_InsertTextString", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_int, ctypes.c_char_p, ctypes.c_size_t], SDL_TTF_BINARY]
TTF_AppendTextString: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_AppendTextString", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_char_p, ctypes.c_size_t], SDL_TTF_BINARY]
TTF_DeleteTextString: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_DeleteTextString", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_int, ctypes.c_int], SDL_TTF_BINARY]

TTF_GetTextSize: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetTextSize", ctypes.c_bool, [SDL_POINTER[TTF_Text], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_TTF_BINARY]

TTF_SubStringFlags: typing.TypeAlias = SDL_TYPE["TTF_SubStringFlags", ctypes.c_uint32]

TTF_SUBSTRING_DIRECTION_MASK, TTF_SUBSTRING_TEXT_START, TTF_SUBSTRING_LINE_START, \
    TTF_SUBSTRING_LINE_END, TTF_SUBSTRING_TEXT_END = 0x000000FF, 0x00000100, 0x00000200, 0x00000400, 0x00000800

class TTF_SubString(ctypes.Structure):
    _fields_ = [
        ("flags", TTF_SubStringFlags),
        ("offset", ctypes.c_int),
        ("length", ctypes.c_int),
        ("line_index", ctypes.c_int),
        ("cluster_index", ctypes.c_int),
        ("rect", SDL_Rect)
    ]

TTF_GetTextSubString: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetTextSubString", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_int, SDL_POINTER[TTF_SubString]], SDL_TTF_BINARY]
TTF_GetTextSubStringForLine: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetTextSubStringForLine", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_int, SDL_POINTER[TTF_SubString]], SDL_TTF_BINARY]
TTF_GetTextSubStringsForRange: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetTextSubStringsForRange", SDL_POINTER[SDL_POINTER[TTF_SubString]], [SDL_POINTER[TTF_Text], ctypes.c_int, ctypes.c_int, SDL_POINTER[ctypes.c_int]], SDL_TTF_BINARY]
TTF_GetTextSubStringForPoint: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetTextSubStringForPoint", ctypes.c_bool, [SDL_POINTER[TTF_Text], ctypes.c_int, ctypes.c_int, SDL_POINTER[TTF_SubString]], SDL_TTF_BINARY]
TTF_GetPreviousTextSubString: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetPreviousTextSubString", ctypes.c_bool, [SDL_POINTER[TTF_Text], SDL_POINTER[TTF_SubString], SDL_POINTER[TTF_SubString]], SDL_TTF_BINARY]
TTF_GetNextTextSubString: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_GetNextTextSubString", ctypes.c_bool, [SDL_POINTER[TTF_Text], SDL_POINTER[TTF_SubString], SDL_POINTER[TTF_SubString]], SDL_TTF_BINARY]
TTF_UpdateText: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_UpdateText", ctypes.c_bool, [SDL_POINTER[TTF_Text]], SDL_TTF_BINARY]
TTF_DestroyText: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_DestroyText", None, [SDL_POINTER[TTF_Text]], SDL_TTF_BINARY]

TTF_CloseFont: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_CloseFont", None, [SDL_POINTER[TTF_Font]], SDL_TTF_BINARY]
TTF_Quit: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_Quit", None, [], SDL_TTF_BINARY]
TTF_WasInit: abc.Callable[..., typing.Any] = SDL_FUNC["TTF_WasInit", ctypes.c_int, [], SDL_TTF_BINARY]


from .__init__ import ctypes, typing, abc, \
    SDL_POINTER, SDL_ENUM, SDL_FUNC, SDL_TYPE, SDL_BINARY

from .SDL_properties import SDL_PropertiesID
from .SDL_iostream import SDL_IOStream

class SDL_Process(ctypes.c_void_p):
    ...

SDL_CreateProcess: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateProcess", SDL_POINTER[SDL_Process], [SDL_POINTER[ctypes.c_char_p], ctypes.c_bool], SDL_BINARY]

SDL_ProcessIO: typing.TypeAlias = SDL_TYPE["SDL_ProcessIO", SDL_ENUM]

SDL_PROCESS_STDIO_INHERITED, SDL_PROCESS_STDIO_NULL, \
    SDL_PROCESS_STDIO_APP, SDL_PROCESS_STDIO_REDIRECT = range(4)

SDL_CreateProcessWithProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateProcessWithProperties", SDL_POINTER[SDL_Process], [SDL_PropertiesID], SDL_BINARY]

SDL_PROP_PROCESS_CREATE_ARGS_POINTER: bytes = "SDL.process.create.args".encode()
SDL_PROP_PROCESS_CREATE_ENVIRONMENT_POINTER: bytes = "SDL.process.create.environment".encode()
SDL_PROP_PROCESS_CREATE_STDIN_NUMBER: bytes = "SDL.process.create.stdin_option".encode()
SDL_PROP_PROCESS_CREATE_STDIN_POINTER: bytes = "SDL.process.create.stdin_source".encode()
SDL_PROP_PROCESS_CREATE_STDOUT_NUMBER: bytes = "SDL.process.create.stdout_option".encode()
SDL_PROP_PROCESS_CREATE_STDOUT_POINTER: bytes = "SDL.process.create.stdout_source".encode()
SDL_PROP_PROCESS_CREATE_STDERR_NUMBER: bytes = "SDL.process.create.stderr_option".encode()
SDL_PROP_PROCESS_CREATE_STDERR_POINTER: bytes = "SDL.process.create.stderr_source".encode()
SDL_PROP_PROCESS_CREATE_STDERR_TO_STDOUT_BOOLEAN: bytes = "SDL.process.create.stderr_to_stdout".encode()
SDL_PROP_PROCESS_CREATE_BACKGROUND_BOOLEAN: bytes = "SDL.process.create.background".encode()

SDL_GetProcessProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetProcessProperties", SDL_PropertiesID, [SDL_POINTER[SDL_Process]], SDL_BINARY]

SDL_PROP_PROCESS_PID_NUMBER: bytes = "SDL.process.pid".encode()
SDL_PROP_PROCESS_STDIN_POINTER: bytes = "SDL.process.stdin".encode()
SDL_PROP_PROCESS_STDOUT_POINTER: bytes = "SDL.process.stdout".encode()
SDL_PROP_PROCESS_STDERR_POINTER: bytes = "SDL.process.stderr".encode()
SDL_PROP_PROCESS_BACKGROUND_BOOLEAN: bytes = "SDL.process.background".encode()

SDL_ReadProcess: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadProcess", ctypes.c_void_p, [SDL_POINTER[SDL_Process], SDL_POINTER[ctypes.c_size_t], SDL_POINTER[ctypes.c_int]], SDL_BINARY]

SDL_GetProcessInput: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetProcessInput", SDL_POINTER[SDL_IOStream], [SDL_POINTER[SDL_Process]], SDL_BINARY]
SDL_GetProcessOutput: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetProcessOutput", SDL_POINTER[SDL_IOStream], [SDL_POINTER[SDL_Process]], SDL_BINARY]

SDL_KillProcess: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_KillProcess", ctypes.c_bool, [SDL_POINTER[SDL_Process], ctypes.c_bool], SDL_BINARY]
SDL_WaitProcess: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WaitProcess", ctypes.c_bool, [SDL_POINTER[SDL_Process], ctypes.c_bool, SDL_POINTER[ctypes.c_int]], SDL_BINARY]
SDL_DestroyProcess: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_DestroyProcess", None, [SDL_POINTER[SDL_Process]], SDL_BINARY]


from .__init__ import ctypes, typing, abc, SDL_POINTER, SDL_VA_LIST, \
    SDL_FUNC_TYPE, SDL_FUNC, SDL_TYPE, SDL_BINARY, SDL_ENUM

from .SDL_properties import SDL_PropertiesID

SDL_IOStatus: typing.TypeAlias = SDL_TYPE["SDL_IOStatus", SDL_ENUM]

SDL_IO_STATUS_READY, SDL_IO_STATUS_ERROR, SDL_IO_STATUS_EOF, SDL_IO_STATUS_NOT_READY, \
    SDL_IO_STATUS_READONLY, SDL_IO_STATUS_WRITEONLY = range(6)

SDL_IOWhence: typing.TypeAlias = SDL_TYPE["SDL_IOWhence", SDL_ENUM]

SDL_IO_SEEK_SET, SDL_IO_SEEK_CUR, SDL_IO_SEEK_END = range(3)

class SDL_IOStreamInterface(ctypes.Structure):
    _fields_ = [
        ("version", ctypes.c_uint32),
        ("size", SDL_FUNC_TYPE["SDL_IOStreamInterface.size", ctypes.c_int64, [ctypes.c_void_p]]),
        ("seek", SDL_FUNC_TYPE["SDL_IOStreamInterface.seek", ctypes.c_int64, [ctypes.c_void_p, ctypes.c_int64, SDL_IOWhence]]),
        ("read", SDL_FUNC_TYPE["SDL_IOStreamInterface.read", ctypes.c_size_t, [ctypes.c_void_p, ctypes.c_void_p, ctypes.c_size_t, SDL_POINTER[SDL_IOStatus]]]),
        ("write", SDL_FUNC_TYPE["SDL_IOStreamInterface.write", ctypes.c_size_t, [ctypes.c_void_p, ctypes.c_void_p, ctypes.c_size_t, SDL_POINTER[SDL_IOStatus]]]),
        ("flush", SDL_FUNC_TYPE["SDL_IOStreamInterface.flush", ctypes.c_bool, [ctypes.c_void_p, SDL_POINTER[SDL_IOStatus]]]),
        ("close", SDL_FUNC_TYPE["SDL_IOStreamInterface.close", ctypes.c_bool, [ctypes.c_void_p]])
    ]

class SDL_IOStream(ctypes.c_void_p):
    ...

SDL_IOFromFile: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_IOFromFile", SDL_POINTER[SDL_IOStream], [ctypes.c_char_p, ctypes.c_char_p], SDL_BINARY]

SDL_PROP_IOSTREAM_WINDOWS_HANDLE_POINTER: bytes = "SDL.iostream.windows.handle".encode()
SDL_PROP_IOSTREAM_STDIO_FILE_POINTER: bytes = "SDL.iostream.stdio.file".encode()
SDL_PROP_IOSTREAM_FILE_DESCRIPTOR_NUMBER: bytes = "SDL.iostream.file_descriptor".encode()
SDL_PROP_IOSTREAM_ANDROID_AASSET_POINTER: bytes = "SDL.iostream.android.aasset".encode()

SDL_IOFromMem: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_IOFromMem", SDL_POINTER[SDL_IOStream], [ctypes.c_void_p, ctypes.c_size_t], SDL_BINARY]

SDL_PROP_IOSTREAM_MEMORY_POINTER: bytes = "SDL.iostream.memory.base".encode()
SDL_PROP_IOSTREAM_MEMORY_SIZE_NUMBER: bytes = "SDL.iostream.memory.size".encode()

SDL_IOFromConstMem: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_IOFromConstMem", SDL_POINTER[SDL_IOStream], [ctypes.c_void_p, ctypes.c_size_t], SDL_BINARY]
SDL_IOFromDynamicMem: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_IOFromDynamicMem", SDL_POINTER[SDL_IOStream], [], SDL_BINARY]

SDL_PROP_IOSTREAM_DYNAMIC_MEMORY_POINTER: bytes = "SDL.iostream.dynamic.memory".encode()
SDL_PROP_IOSTREAM_DYNAMIC_CHUNKSIZE_NUMBER: bytes = "SDL.iostream.dynamic.chunksize".encode()

SDL_OpenIO: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_OpenIO", SDL_POINTER[SDL_IOStream], [SDL_POINTER[SDL_IOStreamInterface], ctypes.c_void_p], SDL_BINARY]
SDL_CloseIO: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CloseIO", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_BINARY]

SDL_GetIOProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetIOProperties", SDL_PropertiesID, [SDL_POINTER[SDL_IOStream]], SDL_BINARY]
SDL_GetIOStatus: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetIOStatus", SDL_IOStatus, [SDL_POINTER[SDL_IOStream]], SDL_BINARY]
SDL_GetIOSize: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetIOSize", ctypes.c_int64, [SDL_POINTER[SDL_IOStream]], SDL_BINARY]

SDL_SeekIO: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SeekIO", ctypes.c_int64, [SDL_POINTER[SDL_IOStream], ctypes.c_int64, SDL_IOWhence], SDL_BINARY]
SDL_TellIO: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_TellIO", ctypes.c_int64, [SDL_POINTER[SDL_IOStream]], SDL_BINARY]
SDL_ReadIO: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadIO", ctypes.c_size_t, [SDL_POINTER[SDL_IOStream], ctypes.c_void_p, ctypes.c_size_t], SDL_BINARY]
SDL_WriteIO: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteIO", ctypes.c_size_t, [SDL_POINTER[SDL_IOStream], ctypes.c_void_p, ctypes.c_size_t], SDL_BINARY]

SDL_IOprintf: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_IOprintf", ctypes.c_size_t, [SDL_POINTER[SDL_IOStream], ctypes.c_char_p, ...], SDL_BINARY]
SDL_IOvprintf: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_IOvprintf", ctypes.c_size_t, [SDL_POINTER[SDL_IOStream], ctypes.c_char_p, SDL_VA_LIST], SDL_BINARY]

SDL_FlushIO: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_FlushIO", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_BINARY]

SDL_LoadFile_IO: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LoadFile_IO", ctypes.c_void_p, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_size_t], ctypes.c_bool], SDL_BINARY]
SDL_LoadFile: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LoadFile", ctypes.c_void_p, [ctypes.c_char_p, SDL_POINTER[ctypes.c_size_t]], SDL_BINARY]

SDL_SaveFile_IO: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SaveFile_IO", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_void_p, ctypes.c_size_t, ctypes.c_bool], SDL_BINARY]
SDL_SaveFile: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SaveFile", ctypes.c_bool, [ctypes.c_char_p, ctypes.c_void_p, ctypes.c_size_t], SDL_BINARY]

SDL_ReadU8: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadU8", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_uint8]], SDL_BINARY]
SDL_ReadS8: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadS8", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_int8]], SDL_BINARY]
SDL_ReadU16LE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadU16LE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_uint16]], SDL_BINARY]
SDL_ReadS16LE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadS16LE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_int16]], SDL_BINARY]
SDL_ReadU16BE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadU16BE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_uint16]], SDL_BINARY]
SDL_ReadS16BE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadS16BE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_int16]], SDL_BINARY]
SDL_ReadU32LE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadU32LE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_uint32]], SDL_BINARY]
SDL_ReadS32LE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadS32LE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_int32]], SDL_BINARY]
SDL_ReadU32BE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadU32BE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_uint32]], SDL_BINARY]
SDL_ReadS32BE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadS32BE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_int32]], SDL_BINARY]
SDL_ReadU64LE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadU64LE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_uint64]], SDL_BINARY]
SDL_ReadS64LE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadS64LE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_int64]], SDL_BINARY]
SDL_ReadU64BE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadU64BE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_uint64]], SDL_BINARY]
SDL_ReadS64BE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ReadS64BE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], SDL_POINTER[ctypes.c_int64]], SDL_BINARY]

SDL_WriteU8: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteU8", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_uint8], SDL_BINARY]
SDL_WriteS8: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteS8", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_int8], SDL_BINARY]
SDL_WriteU16LE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteU16LE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_uint16], SDL_BINARY]
SDL_WriteS16LE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteS16LE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_int16], SDL_BINARY]
SDL_WriteU16BE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteU16BE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_uint16], SDL_BINARY]
SDL_WriteS16BE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteS16BE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_int16], SDL_BINARY]
SDL_WriteU32LE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteU32LE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_uint32], SDL_BINARY]
SDL_WriteS32LE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteS32LE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_int32], SDL_BINARY]
SDL_WriteU32BE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteU32BE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_uint32], SDL_BINARY]
SDL_WriteS32BE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteS32BE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_int32], SDL_BINARY]
SDL_WriteU64LE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteU64LE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_uint64], SDL_BINARY]
SDL_WriteS64LE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteS64LE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_int64], SDL_BINARY]
SDL_WriteU64BE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteU64BE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_uint64], SDL_BINARY]
SDL_WriteS64BE: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WriteS64BE", ctypes.c_bool, [SDL_POINTER[SDL_IOStream], ctypes.c_int64], SDL_BINARY]


from .__init__ import ctypes, typing, abc, \
    SDL_FUNC_TYPE, SDL_ENUM, SDL_FUNC, SDL_TYPE, SDL_BINARY

SDL_PropertiesID: typing.TypeAlias = SDL_TYPE["SDL_PropertiesID", ctypes.c_uint32]
SDL_PropertyType: typing.TypeAlias = SDL_TYPE["SDL_PropertyType", SDL_ENUM]

SDL_PROPERTY_TYPE_INVALID, SDL_PROPERTY_TYPE_POINTER, SDL_PROPERTY_TYPE_STRING, \
    SDL_PROPERTY_TYPE_NUMBER, SDL_PROPERTY_TYPE_FLOAT, SDL_PROPERTY_TYPE_BOOLEAN = range(6)

SDL_GetGlobalProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetGlobalProperties", SDL_PropertiesID, [], SDL_BINARY]
SDL_CreateProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateProperties", SDL_PropertiesID, [], SDL_BINARY]
SDL_CopyProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CopyProperties", ctypes.c_bool, [SDL_PropertiesID, SDL_PropertiesID], SDL_BINARY]
SDL_LockProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LockProperties", ctypes.c_bool, [SDL_PropertiesID], SDL_BINARY]
SDL_UnlockProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_UnlockProperties", None, [SDL_PropertiesID], SDL_BINARY]

SDL_CleanupPropertyCallback: typing.TypeAlias = SDL_FUNC_TYPE["SDL_CleanupPropertyCallback", None, [ctypes.c_void_p, ctypes.c_void_p]]

SDL_SetPointerPropertyWithCleanup: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetPointerPropertyWithCleanup", ctypes.c_bool, [SDL_PropertiesID, ctypes.c_char_p, ctypes.c_void_p, SDL_CleanupPropertyCallback, ctypes.c_void_p], SDL_BINARY]
SDL_SetPointerProperty: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetPointerProperty", ctypes.c_bool, [SDL_PropertiesID, ctypes.c_char_p, ctypes.c_void_p], SDL_BINARY]
SDL_SetStringProperty: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetStringProperty", ctypes.c_bool, [SDL_PropertiesID, ctypes.c_char_p, ctypes.c_char_p], SDL_BINARY]
SDL_SetNumberProperty: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetNumberProperty", ctypes.c_bool, [SDL_PropertiesID, ctypes.c_char_p, ctypes.c_int64], SDL_BINARY]
SDL_SetFloatProperty: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetFloatProperty", ctypes.c_bool, [SDL_PropertiesID, ctypes.c_char_p, ctypes.c_float], SDL_BINARY]
SDL_SetBooleanProperty: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetBooleanProperty", ctypes.c_bool, [SDL_PropertiesID, ctypes.c_char_p, ctypes.c_bool], SDL_BINARY]
SDL_HasProperty: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasProperty", ctypes.c_bool, [SDL_PropertiesID, ctypes.c_char_p], SDL_BINARY]
SDL_GetPropertyType: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetPropertyType", SDL_PropertyType, [SDL_PropertiesID, ctypes.c_char_p], SDL_BINARY]
SDL_GetPointerProperty: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetPointerProperty", ctypes.c_void_p, [SDL_PropertiesID, ctypes.c_char_p, ctypes.c_void_p], SDL_BINARY]
SDL_GetStringProperty: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetStringProperty", ctypes.c_char_p, [SDL_PropertiesID, ctypes.c_char_p, ctypes.c_char_p], SDL_BINARY]
SDL_GetNumberProperty: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetNumberProperty", ctypes.c_int64, [SDL_PropertiesID, ctypes.c_char_p, ctypes.c_int64], SDL_BINARY]
SDL_GetFloatProperty: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetFloatProperty", ctypes.c_float, [SDL_PropertiesID, ctypes.c_char_p, ctypes.c_float], SDL_BINARY]
SDL_GetBooleanProperty: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetBooleanProperty", ctypes.c_bool, [SDL_PropertiesID, ctypes.c_char_p, ctypes.c_bool], SDL_BINARY]
SDL_ClearProperty: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ClearProperty", ctypes.c_bool, [SDL_PropertiesID, ctypes.c_char_p], SDL_BINARY]

SDL_EnumeratePropertiesCallback: typing.TypeAlias = SDL_FUNC_TYPE["SDL_EnumeratePropertiesCallback", None, [ctypes.c_void_p, SDL_PropertiesID, ctypes.c_char_p]]

SDL_EnumerateProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_EnumerateProperties", ctypes.c_bool, [SDL_PropertiesID, SDL_EnumeratePropertiesCallback, ctypes.c_void_p], SDL_BINARY]
SDL_DestroyProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_DestroyProperties", None, [SDL_PropertiesID], SDL_BINARY]


import ctypes, colorsys, time, sdl3, OpenGL.GL as gl

from imgui_bundle.python_backends.sdl3_backend import imgui, SDL3Renderer

@sdl3.SDL_main_func
def main(argc: ctypes.c_int, argv: sdl3.LP_c_char_p) -> ctypes.c_int:
    if not sdl3.SDL_Init(sdl3.SDL_INIT_VIDEO | sdl3.SDL_INIT_EVENTS):
        print(f"Failed to initialize library: {sdl3.SDL_GetError().decode()}.")
        return -1
    
    sdl3.SDL_GL_SetAttribute(sdl3.SDL_GL_CONTEXT_MAJOR_VERSION, 4)
    sdl3.SDL_GL_SetAttribute(sdl3.SDL_GL_CONTEXT_MINOR_VERSION, 6)
    sdl3.SDL_GL_SetAttribute(sdl3.SDL_GL_CONTEXT_PROFILE_MASK, sdl3.SDL_GL_CONTEXT_PROFILE_COMPATIBILITY)
    sdl3.SDL_GL_SetAttribute(sdl3.SDL_GL_CONTEXT_FLAGS, sdl3.SDL_GL_CONTEXT_FORWARD_COMPATIBLE_FLAG)
    window = sdl3.SDL_CreateWindow("Aermoss".encode(), 1600, 900, sdl3.SDL_WINDOW_OPENGL | sdl3.SDL_WINDOW_RESIZABLE)

    if not window:
        print(f"Failed to create window: {sdl3.SDL_GetError().decode()}.", flush = True)
        return -1

    context = sdl3.SDL_GL_CreateContext(window)
    sdl3.SDL_GL_MakeCurrent(window, context)

    if not context:
        print(f"Failed to create context: {sdl3.SDL_GetError().decode()}.", flush = True)
        return -1

    imgui.create_context()
    imgui.get_io().set_ini_filename("")

    renderer = SDL3Renderer(window)
    running, hue, lastTime = True, 0.0, time.time()
    event = sdl3.SDL_Event()

    while running:
        renderer.process_inputs()

        while sdl3.SDL_PollEvent(ctypes.byref(event)):
            renderer.process_event(event)

            match event.type:
                case sdl3.SDL_EVENT_QUIT:
                    running = False

                case sdl3.SDL_EVENT_KEY_DOWN:
                    if event.key.key in [sdl3.SDLK_ESCAPE]:
                        running = False

        lastTime, deltaTime = \
            time.time(), time.time() - lastTime

        hue = (hue + 0.5 * deltaTime) % 360.0
        gl.glClearColor(*colorsys.hsv_to_rgb(hue, 1.0, 0.1), 1.0)
        gl.glClear(gl.GL_COLOR_BUFFER_BIT)

        imgui.new_frame()
        imgui.show_demo_window()
        imgui.end_frame()

        imgui.render()
        renderer.render(imgui.get_draw_data())
        sdl3.SDL_GL_SwapWindow(window)

    renderer.shutdown()
    imgui.destroy_context()

    sdl3.SDL_GL_MakeCurrent(window, None)
    sdl3.SDL_GL_DestroyContext(context)
    sdl3.SDL_DestroyWindow(window)
    sdl3.SDL_Quit()
    return 0


from .__init__ import ctypes, typing, abc, SDL_POINTER, \
    SDL_FUNC_TYPE, SDL_FUNC, SDL_TYPE, SDL_MIXER_BINARY, SDL_ENUM

from .SDL_iostream import SDL_IOStream
from .SDL_audio import SDL_AudioDeviceID, SDL_AudioSpec, SDL_AudioFormat, SDL_AUDIO_S16
from .SDL_version import SDL_VERSIONNUM

SDL_MIXER_MAJOR_VERSION, SDL_MIXER_MINOR_VERSION, SDL_MIXER_MICRO_VERSION = 3, 0, 0
SDL_MIXER_VERSION: int = SDL_VERSIONNUM(SDL_MIXER_MAJOR_VERSION, SDL_MIXER_MINOR_VERSION, SDL_MIXER_MICRO_VERSION)

SDL_MIXER_VERSION_ATLEAST: abc.Callable[[int, int, int], bool] = lambda x, y, z: \
    (SDL_MIXER_MAJOR_VERSION >= x) and (SDL_MIXER_MAJOR_VERSION > x or SDL_MIXER_MINOR_VERSION >= y) and \
        (SDL_MIXER_MAJOR_VERSION > x or SDL_MIXER_MINOR_VERSION > y or SDL_MIXER_MICRO_VERSION >= z)

Mix_Version: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_Version", ctypes.c_int, [], SDL_MIXER_BINARY]

MIX_InitFlags: typing.TypeAlias = SDL_TYPE["MIX_InitFlags", ctypes.c_uint32]

MIX_INIT_FLAC: int = 0x00000001
MIX_INIT_MOD: int = 0x00000002
MIX_INIT_MP3: int = 0x00000008
MIX_INIT_OGG: int = 0x00000010
MIX_INIT_MID: int = 0x00000020
MIX_INIT_OPUS: int = 0x00000040
MIX_INIT_WAVPACK: int = 0x00000080

Mix_Init: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_Init", MIX_InitFlags, [MIX_InitFlags], SDL_MIXER_BINARY]
Mix_Quit: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_Quit", None, [], SDL_MIXER_BINARY]

MIX_CHANNELS: int = 8

MIX_DEFAULT_FREQUENCY: int = 44100
MIX_DEFAULT_FORMAT: int = SDL_AUDIO_S16
MIX_DEFAULT_CHANNELS: int = 2
MIX_MAX_VOLUME: int = 128

class Mix_Chunk(ctypes.Structure):
    _fields_ = [
        ("allocated", ctypes.c_int),
        ("abuf", SDL_POINTER[ctypes.c_uint8]),
        ("alen", ctypes.c_uint32),
        ("volume", ctypes.c_uint8)
    ]

Mix_Fading: typing.TypeAlias = SDL_TYPE["Mix_Fading", SDL_ENUM]

MIX_NO_FADING, MIX_FADING_OUT, MIX_FADING_IN = range(3)

Mix_MusicType: typing.TypeAlias = SDL_TYPE["Mix_MusicType", SDL_ENUM]

MUS_NONE, MUS_WAV, MUS_MOD, MUS_MID, MUS_OGG, MUS_MP3, \
    MUS_FLAC, MUS_OPUS, MUS_WAVPACK, MUS_GME = range(10)

class Mix_Music(ctypes.c_void_p):
    ...

Mix_OpenAudio: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_OpenAudio", ctypes.c_bool, [SDL_AudioDeviceID, SDL_POINTER[SDL_AudioSpec]], SDL_MIXER_BINARY]
Mix_PauseAudio: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_PauseAudio", None, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_QuerySpec: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_QuerySpec", ctypes.c_bool, [SDL_POINTER[ctypes.c_int], SDL_POINTER[SDL_AudioFormat], SDL_POINTER[ctypes.c_int]], SDL_MIXER_BINARY]
Mix_AllocateChannels: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_AllocateChannels", ctypes.c_int, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_LoadWAV_IO: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_LoadWAV_IO", SDL_POINTER[Mix_Chunk], [SDL_POINTER[SDL_IOStream], ctypes.c_bool], SDL_MIXER_BINARY]
Mix_LoadWAV: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_LoadWAV", SDL_POINTER[Mix_Chunk], [ctypes.c_char_p], SDL_MIXER_BINARY]
Mix_LoadMUS: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_LoadMUS", SDL_POINTER[Mix_Music], [ctypes.c_char_p], SDL_MIXER_BINARY]
Mix_LoadMUS_IO: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_LoadMUS_IO", SDL_POINTER[Mix_Music], [SDL_POINTER[SDL_IOStream], ctypes.c_bool], SDL_MIXER_BINARY]
Mix_LoadMUSType_IO: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_LoadMUSType_IO", SDL_POINTER[Mix_Music], [SDL_POINTER[SDL_IOStream], Mix_MusicType, ctypes.c_bool], SDL_MIXER_BINARY]
Mix_QuickLoad_WAV: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_QuickLoad_WAV", SDL_POINTER[Mix_Chunk], [SDL_POINTER[ctypes.c_uint8]], SDL_MIXER_BINARY]
Mix_QuickLoad_RAW: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_QuickLoad_RAW", SDL_POINTER[Mix_Chunk], [SDL_POINTER[ctypes.c_uint8], ctypes.c_uint32], SDL_MIXER_BINARY]
Mix_FreeChunk: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_FreeChunk", None, [SDL_POINTER[Mix_Chunk]], SDL_MIXER_BINARY]
Mix_FreeMusic: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_FreeMusic", None, [SDL_POINTER[Mix_Music]], SDL_MIXER_BINARY]
Mix_GetNumChunkDecoders: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetNumChunkDecoders", ctypes.c_int, [], SDL_MIXER_BINARY]
Mix_GetChunkDecoder: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetChunkDecoder", ctypes.c_char_p, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_HasChunkDecoder: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_HasChunkDecoder", ctypes.c_bool, [ctypes.c_char_p], SDL_MIXER_BINARY]
Mix_GetNumMusicDecoders: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetNumMusicDecoders", ctypes.c_int, [], SDL_MIXER_BINARY]
Mix_GetMusicDecoder: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetMusicDecoder", ctypes.c_char_p, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_HasMusicDecoder: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_HasMusicDecoder", ctypes.c_bool, [ctypes.c_char_p], SDL_MIXER_BINARY]
Mix_GetMusicType: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetMusicType", Mix_MusicType, [SDL_POINTER[Mix_Music]], SDL_MIXER_BINARY]
Mix_GetMusicTitle: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetMusicTitle", ctypes.c_char_p, [SDL_POINTER[Mix_Music]], SDL_MIXER_BINARY]
Mix_GetMusicTitleTag: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetMusicTitleTag", ctypes.c_char_p, [SDL_POINTER[Mix_Music]], SDL_MIXER_BINARY]
Mix_GetMusicArtistTag: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetMusicArtistTag", ctypes.c_char_p, [SDL_POINTER[Mix_Music]], SDL_MIXER_BINARY]
Mix_GetMusicAlbumTag: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetMusicAlbumTag", ctypes.c_char_p, [SDL_POINTER[Mix_Music]], SDL_MIXER_BINARY]
Mix_GetMusicCopyrightTag: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetMusicCopyrightTag", ctypes.c_char_p, [SDL_POINTER[Mix_Music]], SDL_MIXER_BINARY]

Mix_MixCallback: typing.TypeAlias = SDL_FUNC_TYPE["Mix_MixCallback", None, [ctypes.c_void_p, SDL_POINTER[ctypes.c_uint8], ctypes.c_int]]

Mix_SetPostMix: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_SetPostMix", None, [Mix_MixCallback, ctypes.c_void_p], SDL_MIXER_BINARY]
Mix_HookMusic: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_HookMusic", None, [Mix_MixCallback, ctypes.c_void_p], SDL_MIXER_BINARY]

Mix_MusicFinishedCallback: typing.TypeAlias = SDL_FUNC_TYPE["Mix_MusicFinishedCallback", None, []]

Mix_HookMusicFinished: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_HookMusicFinished", None, [Mix_MusicFinishedCallback], SDL_MIXER_BINARY]
Mix_GetMusicHookData: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetMusicHookData", ctypes.c_void_p, [], SDL_MIXER_BINARY]

Mix_ChannelFinishedCallback: typing.TypeAlias = SDL_FUNC_TYPE["Mix_ChannelFinishedCallback", None, [ctypes.c_int]]

Mix_ChannelFinished: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_ChannelFinished", None, [Mix_ChannelFinishedCallback], SDL_MIXER_BINARY]

MIX_CHANNEL_POST: int = -2

Mix_EffectFunc_t: typing.TypeAlias = SDL_FUNC_TYPE["Mix_EffectFunc_t", None, [ctypes.c_int, ctypes.c_void_p, ctypes.c_int, ctypes.c_void_p]]
Mix_EffectDone_t: typing.TypeAlias = SDL_FUNC_TYPE["Mix_EffectDone_t", None, [ctypes.c_int, ctypes.c_void_p]]

Mix_RegisterEffect: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_RegisterEffect", ctypes.c_bool, [ctypes.c_int, Mix_EffectFunc_t, Mix_EffectDone_t, ctypes.c_void_p], SDL_MIXER_BINARY]
Mix_UnregisterEffect: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_UnregisterEffect", ctypes.c_bool, [ctypes.c_int, Mix_EffectFunc_t], SDL_MIXER_BINARY]
Mix_UnregisterAllEffects: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_UnregisterAllEffects", ctypes.c_bool, [ctypes.c_int], SDL_MIXER_BINARY]

MIX_EFFECTSMAXSPEED: bytes = "MIX_EFFECTSMAXSPEED".encode()

Mix_SetPanning: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_SetPanning", ctypes.c_bool, [ctypes.c_int, ctypes.c_uint8, ctypes.c_uint8], SDL_MIXER_BINARY]
Mix_SetPosition: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_SetPosition", ctypes.c_bool, [ctypes.c_int, ctypes.c_int16, ctypes.c_uint8], SDL_MIXER_BINARY]
Mix_SetDistance: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_SetDistance", ctypes.c_bool, [ctypes.c_int, ctypes.c_uint8], SDL_MIXER_BINARY]
Mix_SetReverseStereo: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_SetReverseStereo", ctypes.c_bool, [ctypes.c_int, ctypes.c_int], SDL_MIXER_BINARY]
Mix_ReserveChannels: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_ReserveChannels", ctypes.c_int, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_GroupChannel: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GroupChannel", ctypes.c_bool, [ctypes.c_int, ctypes.c_int], SDL_MIXER_BINARY]
Mix_GroupChannels: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GroupChannels", ctypes.c_bool, [ctypes.c_int, ctypes.c_int, ctypes.c_int], SDL_MIXER_BINARY]
Mix_GroupAvailable: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GroupAvailable", ctypes.c_int, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_GroupCount: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GroupCount", ctypes.c_int, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_GroupOldest: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GroupOldest", ctypes.c_int, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_GroupNewer: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GroupNewer", ctypes.c_int, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_PlayChannel: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_PlayChannel", ctypes.c_int, [ctypes.c_int, SDL_POINTER[Mix_Chunk], ctypes.c_int], SDL_MIXER_BINARY]
Mix_PlayChannelTimed: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_PlayChannelTimed", ctypes.c_int, [ctypes.c_int, SDL_POINTER[Mix_Chunk], ctypes.c_int, ctypes.c_int], SDL_MIXER_BINARY]
Mix_PlayMusic: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_PlayMusic", ctypes.c_bool, [SDL_POINTER[Mix_Music], ctypes.c_int], SDL_MIXER_BINARY]    
Mix_FadeInMusic: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_FadeInMusic", ctypes.c_bool, [SDL_POINTER[Mix_Music], ctypes.c_int, ctypes.c_int], SDL_MIXER_BINARY]
Mix_FadeInMusicPos: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_FadeInMusicPos", ctypes.c_bool, [SDL_POINTER[Mix_Music], ctypes.c_int, ctypes.c_int, ctypes.c_double], SDL_MIXER_BINARY]
Mix_FadeInChannel: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_FadeInChannel", ctypes.c_int, [ctypes.c_int, SDL_POINTER[Mix_Chunk], ctypes.c_int, ctypes.c_int], SDL_MIXER_BINARY]
Mix_FadeInChannelTimed: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_FadeInChannelTimed", ctypes.c_int, [ctypes.c_int, SDL_POINTER[Mix_Chunk], ctypes.c_int, ctypes.c_int, ctypes.c_int], SDL_MIXER_BINARY]
Mix_Volume: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_Volume", ctypes.c_int, [ctypes.c_int, ctypes.c_int], SDL_MIXER_BINARY]
Mix_VolumeChunk: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_VolumeChunk", ctypes.c_int, [SDL_POINTER[Mix_Chunk], ctypes.c_int], SDL_MIXER_BINARY]
Mix_VolumeMusic: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_VolumeMusic", ctypes.c_int, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_GetMusicVolume: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetMusicVolume", ctypes.c_int, [SDL_POINTER[Mix_Music]], SDL_MIXER_BINARY]
Mix_MasterVolume: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_MasterVolume", ctypes.c_int, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_HaltChannel: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_HaltChannel", None, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_HaltGroup: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_HaltGroup", None, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_HaltMusic: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_HaltMusic", None, [], SDL_MIXER_BINARY]
Mix_ExpireChannel: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_ExpireChannel", ctypes.c_int, [ctypes.c_int, ctypes.c_int], SDL_MIXER_BINARY]
Mix_FadeOutChannel: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_FadeOutChannel", ctypes.c_int, [ctypes.c_int, ctypes.c_int], SDL_MIXER_BINARY]
Mix_FadeOutGroup: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_FadeOutGroup", ctypes.c_int, [ctypes.c_int, ctypes.c_int], SDL_MIXER_BINARY]
Mix_FadeOutMusic: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_FadeOutMusic", ctypes.c_bool, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_FadingMusic: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_FadingMusic", Mix_Fading, [], SDL_MIXER_BINARY]
Mix_FadingChannel: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_FadingChannel", Mix_Fading, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_Pause: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_Pause", None, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_PauseGroup: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_PauseGroup", None, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_Resume: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_Resume", None, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_ResumeGroup: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_ResumeGroup", None, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_Paused: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_Paused", ctypes.c_int, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_PauseMusic: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_PauseMusic", None, [], SDL_MIXER_BINARY]
Mix_ResumeMusic: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_ResumeMusic", None, [], SDL_MIXER_BINARY]
Mix_RewindMusic: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_RewindMusic", None, [], SDL_MIXER_BINARY]
Mix_PausedMusic: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_PausedMusic", ctypes.c_bool, [], SDL_MIXER_BINARY]
Mix_ModMusicJumpToOrder: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_ModMusicJumpToOrder", ctypes.c_bool, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_StartTrack: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_StartTrack", ctypes.c_bool, [SDL_POINTER[Mix_Music], ctypes.c_int], SDL_MIXER_BINARY]
Mix_GetNumTracks: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetNumTracks", ctypes.c_int, [SDL_POINTER[Mix_Music]], SDL_MIXER_BINARY]
Mix_SetMusicPosition: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_SetMusicPosition", ctypes.c_bool, [ctypes.c_double], SDL_MIXER_BINARY]
Mix_GetMusicPosition: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetMusicPosition", ctypes.c_double, [SDL_POINTER[Mix_Music]], SDL_MIXER_BINARY]
Mix_MusicDuration: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_MusicDuration", ctypes.c_double, [SDL_POINTER[Mix_Music]], SDL_MIXER_BINARY]
Mix_GetMusicLoopStartTime: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetMusicLoopStartTime", ctypes.c_double, [SDL_POINTER[Mix_Music]], SDL_MIXER_BINARY]
Mix_GetMusicLoopEndTime: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetMusicLoopEndTime", ctypes.c_double, [SDL_POINTER[Mix_Music]], SDL_MIXER_BINARY]
Mix_GetMusicLoopLengthTime: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetMusicLoopLengthTime", ctypes.c_double, [SDL_POINTER[Mix_Music]], SDL_MIXER_BINARY]
Mix_Playing: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_Playing", ctypes.c_int, [ctypes.c_int], SDL_MIXER_BINARY]
Mix_PlayingMusic: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_PlayingMusic", ctypes.c_bool, [], SDL_MIXER_BINARY]
Mix_SetSoundFonts: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_SetSoundFonts", ctypes.c_bool, [ctypes.c_char_p], SDL_MIXER_BINARY]
Mix_GetSoundFonts: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetSoundFonts", ctypes.c_char_p, [], SDL_MIXER_BINARY]

Mix_EachSoundFontCallback: typing.TypeAlias = SDL_FUNC_TYPE["Mix_EachSoundFontCallback", ctypes.c_bool, [ctypes.c_char_p, ctypes.c_void_p]]

Mix_EachSoundFont: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_EachSoundFont", ctypes.c_bool, [Mix_EachSoundFontCallback, ctypes.c_void_p], SDL_MIXER_BINARY]
Mix_SetTimidityCfg: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_SetTimidityCfg", ctypes.c_bool, [ctypes.c_char_p], SDL_MIXER_BINARY]
Mix_GetTimidityCfg: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetTimidityCfg", ctypes.c_char_p, [], SDL_MIXER_BINARY]
Mix_GetChunk: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_GetChunk", SDL_POINTER[Mix_Chunk], [ctypes.c_int], SDL_MIXER_BINARY]
Mix_CloseAudio: abc.Callable[..., typing.Any] = SDL_FUNC["Mix_CloseAudio", None, [], SDL_MIXER_BINARY]


from .__init__ import sdl3, ctypes, TEST_RegisterFunction

@TEST_RegisterFunction(["Linux", "Darwin", "Windows"])
def TEST_SDL_Init() -> None:
    assert sdl3.SDL_Init(sdl3.SDL_INIT_EVENTS), sdl3.SDL_GetError().decode()
    assert sdl3.SDL_WasInit(0) & sdl3.SDL_INIT_EVENTS, "Failed to initialize subsystem."
    assert (error := sdl3.SDL_GetError()) == "".encode(), error.decode()
    sdl3.SDL_Quit()

@TEST_RegisterFunction(["Linux", "Darwin", "Windows"])
def TEST_SDL_InitSubSystem() -> None:
    assert sdl3.SDL_Init(0), sdl3.SDL_GetError().decode()
    assert sdl3.SDL_InitSubSystem(sdl3.SDL_INIT_EVENTS), sdl3.SDL_GetError().decode()
    assert (error := sdl3.SDL_GetError()) == "".encode(), error.decode()
    assert sdl3.SDL_WasInit(0) & sdl3.SDL_INIT_EVENTS, "Failed to initialize subsystem."
    sdl3.SDL_QuitSubSystem(sdl3.SDL_INIT_EVENTS)
    sdl3.SDL_Quit()

@TEST_RegisterFunction(["Linux", "Darwin", "Windows"])
def TEST_SDL_IsMainThread() -> None:
    assert sdl3.SDL_IsMainThread(), sdl3.SDL_GetError().decode()

@sdl3.SDL_MainThreadCallback
def callback(data: ctypes.c_void_p) -> None:
    ctypes.cast(data, sdl3.SDL_POINTER[ctypes.c_bool])[0] = True

@TEST_RegisterFunction(["Linux", "Darwin", "Windows"])
def TEST_SDL_RunOnMainThread() -> None:
    data = ctypes.pointer(ctypes.c_bool(False))
    assert sdl3.SDL_RunOnMainThread(callback, ctypes.cast(data, ctypes.c_void_p), True), sdl3.SDL_GetError().decode()
    assert data[0], "Failed to run on main thread."